from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal import logger
from portal.database.models import ClientType
from portal.rest import rest_helpers
from portal.rest.schemas import SCHEMAS
from portal.service import permissions, general
from portal.service.secrets import create_secret, delete_secret

client_namespace = Namespace('client')
clients_namespace = Namespace('clients')

log = logger.get_logger(__name__)


@client_namespace.route('')
class ClientController(Resource):
    @jwt_required
    # @users_namespace.response(200, 'List of users', model=users_schema)
    def get(self):
        perm_service = permissions.PermissionsService()
        client = perm_service.client_owner
        schema = SCHEMAS.user() if client.type == ClientType.USER else SCHEMAS.worker()
        return schema.dump(client)[0], 200


@clients_namespace.route('/<string:cid>/secrets')
@clients_namespace.param('wid', "Client's id")
@clients_namespace.response(404, 'Client not found')
class ClientSecretsController(Resource):
    @jwt_required
    @clients_namespace.response(200, 'List client secrets')
    def get(self, cid: str):
        permissions.PermissionsService().require.sysadmin_or_self(cid)
        client = general.find_client(cid)
        return SCHEMAS.dump('secrets', client.secrets)

    @jwt_required
    # @workers_namespace.response(201, 'Created worker secret', model=secret_schema)
    def post(self, cid: str):
        permissions.PermissionsService().require.sysadmin_or_self(cid)
        data = rest_helpers.parse_request_data(action='create', resource='secret')
        client = general.find_client(cid)
        new_secret, value = create_secret(client, **data)
        return {'id': new_secret.id, 'value': value}, 201


@clients_namespace.route('/<string:cid>/secrets/<string:sid>')
@clients_namespace.param('wid', "Client's id")
@clients_namespace.param('sid', "Secret id")
@clients_namespace.response(404, 'Client not found')
class ClientSecretController(Resource):
    @jwt_required
    @clients_namespace.response(200, 'Client secret detail')
    def get(self, cid: str, sid: str):
        permissions.PermissionsService().require.sysadmin()
        client = general.find_client(cid)
        secret = general.find_secret(client, sid)
        return SCHEMAS.dump('secret', secret)

    @jwt_required
    @clients_namespace.response(204, 'Client secret deleted')
    def delete(self, cid: str, sid: str):
        permissions.PermissionsService().require.sysadmin_or_self(cid)
        worker = general.find_client(cid)
        delete_secret(worker, sid)
        return '', 204

    @jwt_required
    @clients_namespace.response(204, 'Client secret updated')
    @clients_namespace.response(403, 'Not allowed to update client secret')
    def put(self, cid: str, sid: str):
        permissions.PermissionsService().require.sysadmin_or_self(cid)

        data = rest_helpers.parse_request_data(action='update', resource='secret', partial=True)

        client = general.find_client(cid)
        secret = general.find_secret(client, sid)
        general.update_entity(secret, data, ['name', 'expires_at'])
        return '', 204



