import json

import pytest

from portal.database.models import Client, Secret
from tests.rest import utils


@pytest.fixture
def student_credentials():
    return json.dumps({
        "type": "username_password",
        "identifier": "student1",
        "secret": "123456"
    })


@pytest.fixture
def worker_credentials():
    return json.dumps({
        "type": "secret",
        "identifier": "executor",
        "secret": "executor_secret"
    })


def test_client_detail_using_student_cred(client, student_credentials):
    path = f'/client'
    response = utils.make_request(
        client, path, 'get', credentials=student_credentials)
    print(f"Response: {response.data}")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    data = response.json
    assert data['username'] == 'student1'
    assert data['type'] == 'ClientType.USER'


def test_client_detail_using_executor_cred(client, worker_credentials):
    path = f'/client'
    response = utils.make_request(
        client, path, 'get', credentials=worker_credentials)
    print(f"Response: {response.data}")
    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    data = response.json
    assert data['name'] == 'executor'
    assert data['type'] == 'ClientType.WORKER'


def test_create_secret(client):
    instance = Client.query.filter_by(codename="executor").first()
    request_dict = dict(name="new_secret")
    response = utils.make_request(client, f'/clients/{instance.id}/secrets',
                                  headers={"content-type": "application/json"},
                                  data=json.dumps(request_dict),
                                  method='post')

    assert response.status_code == 201
    assert response.mimetype == 'application/json'
    parsed = response.json
    assert parsed['value'] is not None
    assert parsed['id'] is not None

    secrets = Secret.query.filter_by(client=instance)
    assert len(secrets.all()) == 2
    assert secrets.filter_by(name='new_secret').first()


def test_list_secret(client):
    instance = Client.query.filter_by(codename="executor").first()
    response = utils.make_request(client, f'/clients/{instance.id}/secrets',
                                  headers={"content-type": "application/json"},
                                  method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    secrets = utils.extract_data(response)
    assert len(secrets) == 1


def test_delete_secret(client):
    worker = Client.query.filter_by(codename="executor").first()
    secret = Secret.query.filter_by(client=worker).first()
    response = utils.make_request(client, f'/clients/{worker.id}/secrets/{secret.id}',
                                  headers={"content-type": "application/json"},
                                  method='delete')

    assert response.status_code == 204

    assert Secret.query.filter_by(client=worker).count() == 0


def test_read_secret(client):
    instance = Client.query.filter_by(codename="executor").first()
    secret = Secret.query.filter_by(client=instance).first()
    response = utils.make_request(client, f'/clients/{instance.id}/secrets/{secret.id}',
                                  headers={"content-type": "application/json"},
                                  method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    response_secret = utils.extract_data(response)
    assert response_secret['id'] == secret.id
    assert response_secret['name'] == secret.name

    assert Secret.query.filter_by(client=instance).count() == 1


def test_update_secret(client):
    instance = Client.query.filter_by(codename="executor").first()
    request_dict = dict(name="new_name")
    secret = Secret.query.filter_by(client=instance).first()
    response = utils.make_request(client, f'/clients/{instance.id}/secrets/{secret.id}',
                                  headers={"content-type": "application/json"},
                                  data=json.dumps(request_dict),
                                  method='put')

    assert response.status_code == 204
    updated_secret = Secret.query.filter_by(client=instance).first()
    assert updated_secret.id == secret.id
    assert updated_secret.name == "new_name"

    assert Secret.query.filter_by(client=instance).count() == 1
